<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2019.3/2027974
 */

namespace Projects\View\Helper;

use Application\Config\ConfigManager;
use Application\Connection\ConnectionFactory;
use Application\Model\IModelDAO;
use Application\Permissions\Permissions;
use Projects\Model\Project as ProjectModel;
use Application\View\Helper\AbstractHelper;

class ProjectToolbar extends AbstractHelper
{
    /**
     * Returns the markup for a project toolbar.
     *
     * @param ProjectModel|string $project the project to render toolbar for
     * @param string              $readme  the project readme
     * @return  string              markup for the project toolbar
     * @throws \Application\Config\ConfigException
     */
    public function __invoke($project, $readme = '')
    {
        return $this->buildToolBar($this->getView(), $project, $readme);
    }

    /**
     * Build the Toolbar for project page
     *
     * @param string       $view      Page view
     * @param ProjectModel $project   Project the toolbar is being built for.
     * @param string       $readme    The project readme
     * @return string    Return a html element
     * @throws \Application\Config\ConfigException
     */
    public function buildToolBar($view, $project, $readme)
    {
        $services    = $this->services;
        $projectDAO  = $services->get(IModelDAO::PROJECT_DAO);
        $permissions = $services->get(Permissions::PERMISSIONS);
        $event       = $services->get('Application')->getMvcEvent();
        $route       = $event->getRouteMatch()->getMatchedRouteName();
        $mode        = $event->getRouteMatch()->getParam('mode');
        $config      = $services->get(ConfigManager::CONFIG);

        // get project model if project is passed via id
        if (!$project instanceof ProjectModel) {
            $p4Admin = $services->get(ConnectionFactory::P4_ADMIN);
            $project = $projectDAO->fetch($project, $p4Admin);
        }

        $links = [];

        // Check if readme is enabled.
        $readmeEnabled = $this->fetchOverviewLink($view, $route, $project->getId(), $readme);
        if (is_array($readmeEnabled)) {
            $links[] = $readmeEnabled;
        }

        // declare project links
        $links[] =  [
            'label'  => 'Activity',
            'url'    => $view->url('project-activity', ['project' => $project->getId()]),
            'active' => $route === 'project-activity' || $route === 'activity',
            'class'  => 'activity-link'
        ];
        $links[] = [
            'label'  => 'Reviews',
            'url'    => $view->url('project-reviews', ['project' => $project->getId()]),
            'active' => $route === 'project-reviews' || $route === 'review',
            'class'  => 'review-link'
        ];

        // add links to view projects files and history if projects has (any) branches
        if (count($project->getBranches())) {
            $links[] = [
                'label'  => 'Files',
                'url'    => $view->url('project-browse', ['project' => $project->getId(), 'mode' => 'files']),
                'active' => $route === 'project-browse' && $mode === 'files',
                'class'  => 'browse-link'
            ];
            $links[] = [
                'label'  => 'Commits',
                'url'    => $view->url('project-browse', ['project' => $project->getId(), 'mode' => 'changes']),
                'active' => $route === 'project-browse' && $mode === 'changes',
                'class'  => 'history-link'
            ];
        }

        // add a jobs link if project has a job filter set.
        if (trim($project->get('jobview'))) {
            $links[] = [
                'label'  => 'Jobs',
                'url'    => $view->url('project-jobs', ['project' => $project->getId()]),
                'active' => $route === 'project-jobs',
                'class'  => 'jobs'
            ];
        }

        // Check if view settings is setup
        $hasOwner  = ['admin', 'owner'  => $project];
        $noOwner   = ['admin', 'member' => $project];
        $allowView = ConfigManager::getValue($config, ConfigManager::PROJECTS_ALLOW_VIEW_SETTINGS);
        if ($allowView) {
            $hasOwner = array_merge($hasOwner, ['member' => $project]);
        }

        // add project settings link if user has permission
        $canEdit = $project->hasOwners()
            ? $permissions->isOne($hasOwner)
            : $permissions->isOne($noOwner);
        if ($canEdit) {
            $links[] = [
                'label'  => 'Settings',
                'url'    => $view->url('edit-project', ['project' => $project->getId()]),
                'active' => $route === 'edit-project',
                'class'  => 'settings'
            ];
        }

        // render list of links
        $list = '';
        foreach ($links as $link) {
            $list .= '<li class="' . ($link['active'] ? 'active' : '') . '">'
                .  '<a href="' . $link['url'] . '" class="' . $link['class'] . '">'
                . $view->te($link['label'])
                . '</a>'
                .  '</li>';
        }

        // render project toolbar
        $name        = $view->escapeHtml($project->getName());
        $url         = $view->url('project', ['project' => $project->getId()]);
        $privateIcon = $project->isPrivate()
            ? '<i class="icon-eye-close private-project-icon" title="' . $view->te('Private Project') . '"></i>'
            : '';
        return '<div class="profile-navbar project-navbar navbar" data-project="' . $project->getId() . '">'
            . ' <div class="navbar-inner">'
            .    $privateIcon
            . '  <a class="brand force-wrap" href="' . $url . '">' . $name . '</a>'
            . '  <ul class="nav">' . $list . '</ul>'
            . ' </div>'
            . '</div>';
    }

    /**
     * Check if the readme is not disabled and it has content
     *
     * @param string $view         View of page
     * @param string $route        Route Swarm page is on
     * @param string $projectId    Project Id working on
     * @param string $readme       Readme content.
     * @return array|null   return the link in the array otherwise null.
     */
    public function fetchOverviewLink($view, $route, $projectId, $readme)
    {
        $link = null;
        if (!empty($readme)) {
            $link = [
                'label'  => $view->te('Overview'),
                'url'    => $view->url('project', ['project' => $projectId]),
                'active' => $route === 'project',
                'class'  => 'overview-link'
            ];
        }
        return $link;
    }
}
